#!/bin/bash

LOGDIR="/tmp/buildlog/`id -u -n`"

TYPE="$1";     shift
NAME="$1";     shift
VERSION="$1";  shift
REVISION="$1"; shift

if [ -z "$TYPE" ] || [ -z "$NAME" ] || [ -z "$VERSION" ] || [ -z "$REVISION" ] || [ -z "$1" ]; then
	echo "usage: $0 <type> <name> <version> <revision> <command> [command-args]"
	exit 0
fi

LOGDIR="$LOGDIR/$NAME-$VERSION-$REVISION"
LOGFILE="$LOGDIR/$TYPE.log"
FAILEDFILE="$LOGDIR/$TYPE.failed"

mkdir -p "$LOGDIR" || :
if [ ! -d "$LOGDIR" ] || [ ! -w "$LOGDIR" ]; then
	"$@"
	exit $?
fi

check_for_failed() {
	if [ -f "$FAILEDFILE" ]; then
		echo "$NAME-$VERSION-$REVISION $TYPE failed!"
		rm -rf "$FAILEDFILE"
		exit 1
	fi
}

onint() {
	trap SIGINT
	check_for_failed
	kill -SIGINT $$
}

touch_failed() {
	touch "$FAILEDFILE"
}

trap onint SIGINT
trap onint SIGQUIT
trap SIGTERM

rm -rf "$FAILEDFILE"

date > "$LOGFILE"
(trap touch_failed SIGINT; trap touch_failed SIGQUIT; "$@" || touch_failed) 2>&1 | tee -a "$LOGFILE" || touch_failed
[ -f "config.log" ] && cp config.log "$LOGDIR/config.log"

check_for_failed
